@@ -9,6 +9,7 @@
  * psx_syscall()-like function that leveraged the nptl:setxid
  * mechanism to synchronize thread state over the whole process.
  */
+
 #ifndef _GNU_SOURCE
 #define _GNU_SOURCE
 #endif
@@ -28,6 +29,9 @@
 #include <sys/types.h>
 #include <unistd.h>
 
+/* Not reliably defined by *libc so, alias the direct syscall. */
+#define _psx_gettid() syscall(SYS_gettid)
+
 #include "psx_syscall.h"
 
 #ifdef _PSX_DEBUG_MEMORY
@@ -35,13 +39,13 @@
 static void *_psx_calloc(const char *file, const int line,
 			 size_t nmemb, size_t size) {
     void *ptr = calloc(nmemb, size);
-    fprintf(stderr, "psx:%d:%s:%d: calloc(%ld, %ld) -> %p\n", gettid(),
+    fprintf(stderr, "psx:%d:%s:%d: calloc(%ld, %ld) -> %p\n", _psx_gettid(),
 	    file, line, (long int)nmemb, (long int)size, ptr);
     return ptr;
 }
 
 static void _psx_free(const char *file, const int line, void *ptr) {
-    fprintf(stderr, "psx:%d:%s:%d: free(%p)\n", gettid(), file, line, ptr);
+    fprintf(stderr, "psx:%d:%s:%d: free(%p)\n", _psx_gettid(), file, line, ptr);
     return free(ptr);
 }
 
@@ -287,7 +291,7 @@ static void psx_posix_syscall_actor(int signum, siginfo_t *info, void *ignore) {
      * communicate the result of the thread's attempt to perform the
      * syscall.
      */
-    long tid = gettid();
+    long tid = _psx_gettid();
 
     psx_lock();
     psx_thread_ref_t *ref = &psx_tracker.map[psx_mix(tid) & psx_tracker.map_mask];
@@ -410,10 +414,11 @@ static long int __psx_immediate_syscall(long int syscall_nr,
 
 #define BUF_SIZE 4096
 
-struct psx_linux_dirent {
-    unsigned long d_ino;
-    off_t d_off;
+struct psx_linux_dirent64 {
+    long long d_ino;
+    long long d_off;
     unsigned short d_reclen;
+    unsigned char d_type;
     char d_name[];
 };
 
@@ -470,7 +475,7 @@ long int __psx_syscall(long int syscall_nr, ...) {
     memset(psx_tracker.map, 0,
 	   psx_tracker.map_entries*sizeof(psx_thread_ref_t));
 
-    long self = gettid(), sweep = 1;
+    long self = _psx_gettid(), sweep = 1;
     int some, incomplete, mismatch = 0, verified = 0;
     do {
 	incomplete = 0;  /* count threads to return from signal handler */
@@ -486,11 +491,11 @@ long int __psx_syscall(long int syscall_nr, ...) {
 
 	for (;;) {
 	    char buf[BUF_SIZE];
-	    size_t nread = syscall(SYS_getdents, fd, buf, BUF_SIZE);
+	    size_t nread = syscall(SYS_getdents64, fd, buf, BUF_SIZE);
 	    if (nread == 0) {
 		break;
 	    } else if (nread < 0) {
-		perror("getdents failed");
+		perror("getdents64 failed");
 		kill(psx_tracker.pid, SIGKILL);
 	    }
 
@@ -499,10 +504,10 @@ long int __psx_syscall(long int syscall_nr, ...) {
 	    for (offset = 0; offset < nread; offset += reclen) {
 		/* deal with potential unaligned reads */
 		memcpy(&reclen, buf + offset +
-		       offsetof(struct psx_linux_dirent, d_reclen),
+		       offsetof(struct psx_linux_dirent64, d_reclen),
 		       sizeof(reclen));
 		char *dir = (buf + offset +
-			     offsetof(struct psx_linux_dirent, d_name));
+			     offsetof(struct psx_linux_dirent64, d_name));
 		long tid = atoi(dir);
 		if (tid == 0 || tid == self) {
 		    continue;
